/* CreateThumbnail.java - hades.manager.CreateThumbnail
 *
 * an utility class to construct a GIF image for a given HADES design
 * or component
 *
 * 02.03.01 - first version, code snips taken from Colibri ImportCommand
 * 01.07.98 - original version by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager;

import java.io.*;
import java.awt.*;

import hades.gui.*;
import hades.models.Design;
import jfig.utils.SetupManager;
import GIFEncoder.*;

/**
 * a class to generate .gif thumbnails for display in the Colibri browser
 * imagePanel. Naturally, the images may also be user for other 
 * purposes...
 */
public class CreateThumbnail {

  static final int   DESIGN_THUMBNAIL = 3;
  static final int   DESIGN_SYMBOL_THUMBNAIL = 4;
  static final int   SIMOBJ_SYMBOL_THUMBNAIL = 5;

  int width  = 300;
  int height = 225;


  Editor      editor;
  GIFEncoder  gifEncoder;


  public CreateThumbnail( Editor editor ) {
    this.editor = editor;

    // advanced feature, but usually looks for downscaled drawings...
    //
    //if (jfig.objects.FigAttribs.enableJava2D) {
    //  editor.getObjectCanvas().getOptions2D().requestAntiAliasing( true );
    //}
  }


  public void setDimension( int width, int height ) {
    this.width = width; 
    this.height = height;
  }


  public void createThumbnailForDesign( String filename ) throws Exception {
    msg( "...creating thumbnail for design file: " + filename );
    String outfilename = 
      filename.substring( 0, filename.lastIndexOf( '.' )) + ".gif";

    msg( "...loading the design..." );
    editor.doOpenDesign( filename, false ); // check unsaved changes

    createEncodeThumbnail( outfilename );
  }




  public void createThumbnailForSubdesign( String filename ) {
    msg( "...creating thumbnail for subdesign symbol: " + filename );
    String outfilename = 
      filename.substring( 0, filename.lastIndexOf( '.' )) + ".gif";

    msg( "...loading the subdesign..." );
    String designName = filename;
    editor.doStartNewDesign( false ); // don't check for changes
    editor.setCommand( new CreateSubdesignCommand( editor, designName ));
    editor.getCommand().setPosition( new Point( 0, 0 ), new Point( 0, 0 ));
    createEncodeThumbnail( outfilename );

  }

  public void createThumbnailForSimObject( String filename ) {
    msg( "...creating thumbnail for simobject symbol: " + filename );
    String className = filename.
                           substring( 0, filename.lastIndexOf( '.' )).
                             replace( '/', '.' );
    msg( "...classname is: " + className );

    String outfilename = 
      filename.substring( 0, filename.lastIndexOf( '.' )) + ".gif";


    editor.doStartNewDesign( false ); // don't check for changes
    editor.setCommand( new CreateCommand( editor, className ));
    editor.getCommand().setPosition( new Point( 0, 0 ), new Point( 0, 0 ));

    createEncodeThumbnail( outfilename );
  }


  private void createEncodeThumbnail( String outfilename ) {
    try {
      msg( "...drawing with zoom fit..." );
      Image im = editor.doCreateThumbnail( width, height );

      msg( "...encoding GIF..." );
      GIFEncoder encoder = new GIFEncoder( im );

      msg( "...writing GIF to file: " + outfilename );
      FileOutputStream giffile = new FileOutputStream( outfilename );
      OutputStream of = new BufferedOutputStream( giffile );
      encoder.Write( of );
      giffile.close();
      msg( "...ok." );
    }
    catch( Exception e ) {
      msg( "-E- internal: " + e );
    }
  }




  public static void usage() {
    msg( "Usage: java hades.manager.CreateThumbnail [mode] [options] [files]\n"
    + "where mode is one of:\n"
    + "-hds                  load design, generate image for it\n"
    + "-sym                  create thumbnail for subdesign\n"
    + "-obj                  create thumbnail for simobject symbol\n"
    + "where options include:\n"
    + "-width pixels         default: 300\n"
    + "-height pixels        default: 225\n"
    + "and files is a list of .hds files\n"
    + "Example(s):\n"
    + "java hades.manager.CreateThumbnail -hds -width 400 examples/foo.hds \n"
    + "java hades.manager.CreateThumbnail -sym -width 400 -height 300 bar.hds\n"
    + "java hades.manager.CreateThumbnail -obj -models/gates/Inv.class \n"
    );
  }


  public static void msg( String s ) {
    System.out.println( s );
  }


  public static void main( String argv[] ) {
    int width = 300, height = 225, mode = DESIGN_THUMBNAIL;
    try {
      if      ("-hds".equals( argv[0] )) mode = DESIGN_THUMBNAIL;
      else if ("-sym".equals( argv[0] )) mode = DESIGN_SYMBOL_THUMBNAIL;
      else if ("-obj".equals( argv[0] )) mode = SIMOBJ_SYMBOL_THUMBNAIL;
      else throw new Exception( "invalide mode argument: " + argv[0] );

      int i = 1;
      if ("-width" .equals( argv[i] )) {
         width  = Integer.parseInt( argv[i+1] );
         i += 2;
      } 
      if ("-height".equals( argv[i] )) {
         height = Integer.parseInt( argv[i+1] );
         i += 2;
      }

     
      jfig.utils.ExceptionTracer.setEnabled( false );
      SetupManager.loadGlobalProperties( "hades/.hadesrc" );
      SetupManager.loadUserProperties ( ".hadesrc" );

      Editor  editor = new Editor();
      CreateThumbnail cifd = new CreateThumbnail( editor );
      cifd.setDimension( width, height );
      for( ; i < argv.length; i++ ) {
        try {
          switch( mode ) {
            case DESIGN_THUMBNAIL:         
                     cifd.createThumbnailForDesign( argv[i] );
                     break;
            case DESIGN_SYMBOL_THUMBNAIL:  
                     cifd.createThumbnailForSubdesign( argv[i] );
                     break;
            case SIMOBJ_SYMBOL_THUMBNAIL:  
                     cifd.createThumbnailForSimObject( argv[i] );
          } 
        }
        catch( Exception e ) {
          msg( "-W- failed for " + argv[i] );
          msg( "-W- exception is: " + e );
        }
      }
      System.exit( 0 );
    }
    catch( Exception e ) {
      usage();
    }
  }
}
