/* TreeManager.java - hades.manager.TreeManager
 *
 * class to build the simobject/design tree 
 * for the local filesystem, JAR/ZIP files, or model servers
 *
 * 19.03.01 - registerZipFile()
 * 15.03.01 - fix building MS-DOS filenames
 * 03.03.01 - getRootNode(), getFullPathname(), getGifPathname()
 * 01.03.01 - new package hades.manager.tree, corresponding changes
 * 28.02.01 - build tree from JAR/ZIP files
 * 27.02.01 - rename to TreeManager, integrated to Hades 0.87, FNH
 * 
 * 01.07.98 - original TreeConstructor by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager;


import java.io.*;
import java.util.*;
import java.util.zip.*;

import javax.swing.*;          // needed for displayTree only
import javax.swing.tree.*;

import hades.manager.tree.*;


/**
 * The class responsible to build and manage the simobject/design tree
 * for Colibri.
 * This class also collects (almost) all methods to update the tree.
 */
public class TreeManager
{

  private SortedTreeNode rootNode;



  public TreeManager()
  {
    super();
  }


  public void setRootNode( SortedTreeNode root ) {
    rootNode = root;
  }


  public SortedTreeNode getRootNode() {
    return rootNode;
  }



  /**
   * traverse the file structure, starting from 'path',
   * and build a tree with all directories and files of the following types:
   * .hds design files, 
   * .clb simobject component info files,
   * .rom memory contents, 
   * .fsm finite state machines, 
   * .py  python scripts.
   * .txt text files
   * README files
   *
   * On non-Unix platforms, we convert external filenames from their native
   * filename separator chars like '\' to unix '/' chars.
   * Returns whether files were added to the 'node' (sub)tree
   */
  public boolean traverseFileTree( File path, SortedTreeNode node ) {
    if (path == null) return false;
    if (!path.isDirectory()) return false;

    // no files so far
    boolean show = false;

    // list directory contents
    String[] list = path.list();
    if (list == null) return false; // empty directory

    for( int i=0; i < list.length; i++ ) {

      File file = new File( path, list[i] );
      //
      // do this in the tree node constructors 
      String filename = file.getPath().replace( '\\', '/' );

      if (!file.canRead()) {
        msg( "-W- TreeManager: no read access: " + file.getAbsolutePath() );
      }
      else 
      if (file.isDirectory()) { // recurse into subtree
        SortedTreeNode directoryNode = new DirectoryTreeNode();
        directoryNode.setUserObject( stripDirectoryNames(filename) ); // FIXME

        if (traverseFileTree( file, directoryNode )) { // not empty
          show = true; // we're not empty because the subtree is not
          node.add( directoryNode );
        }
      }
      else if (isDesignFile(filename)) {
        //TNode designfile = new TNode( new ComponentInfo( file ), node );
        SortedTreeNode fileNode = createLeafNode( 
                                     stripDirectoryNames(filename) ); // FIXME
        show = true;
        node.add( fileNode );
      }
    }
    return show;
  }


  public void sortIntoTree( SortedTreeNode parent, 
                            SortedTreeNode child ) {
    boolean isLeaf = child.isLeaf();

    int n = parent.getChildCount();

  }


  public boolean isDesignFile( String name ) {
    return (   name.endsWith( "README" )
            || name.endsWith( ".hds" ) 
            || name.endsWith( ".clb" )
            || name.endsWith( ".asm" )
            || name.endsWith( ".py" )
            || name.endsWith( ".pyc" )
            || name.endsWith( ".rom" ));
  }



  /**
   * assuming a '/' separated full "path+file+extension" name, 
   * strip off the path name and return the file name.
   * <p>
   * If the name already ends with a '/', we return the last
   * 
     <pre>
     For example:
     "/usr/local/hades/models/gates/And2.hds" -> "And2.hds"
     "/usr/local/hades/models/gates"          -> "gates"
     "/usr/local/"                            -> "local"
     "/"                                      -> ""
     "e:/java/hades/models/io/ClockGen.hds"   -> "ClockGen.hds"
     "c:/"                                    -> "c:"
     </pre>
   */
  public String stripDirectoryNames( String pathname ) {
    if (pathname == null) return null;

    // strip off trailing '/', if any
    //
    if (pathname.charAt( pathname.length()-1 ) == '/') {
       pathname = pathname.substring( 0, pathname.length()-1 );
    }

    int pos = pathname.lastIndexOf( '/' );
    if (pos <= 0) return pathname;
    else          return pathname.substring( pos+1, pathname.length() );
  }


  /**
   * // FIXME: should use the Factory pattern here: TreeNodeFactory.create...
   */
  public SortedTreeNode createLeafNode( String name ) {
    SortedTreeNode node = null;
   
    if (name.endsWith( "README" )) {
       node = new ReadmeFileTreeNode();
    } 
    else if (name.endsWith( ".hds" )) {
       node = new DesignFileTreeNode();
    }
    else if (name.endsWith( ".clb" )) {
       node = new SimObjectTreeNode();
    }
    else if (name.endsWith( ".py" )) {
       node = new PythonFileTreeNode();
    }
    else if (name.endsWith( ".asm" )) {
       node = new AssemblerFileTreeNode();
    }
    else if (name.endsWith( ".rom" )) {
       node = new TextFileTreeNode();
    }
    else {
       node = new SortedTreeNode();
    }
    node.setUserObject( name );
    return node;
  }


  /* ********************************************************************* */
  /* ********************************************************************* */
  /* **** JAR/ZIP stuff ************************************************** */
  /* ********************************************************************* */
  /* ********************************************************************* */


  Hashtable pathToNodeTable = null; // map pathnames to tree nodes


  public ZipRootTreeNode traverseZip( ZipFile zipfile ) {

    ZipRootTreeNode ziproot = new ZipRootTreeNode();
    ziproot.setZipFile( zipfile );

    pathToNodeTable = new Hashtable();
    pathToNodeTable.put( "", ziproot );
   

    Enumeration e = zipfile.entries();
    while( e.hasMoreElements()) {
      ZipEntry     entry = (ZipEntry) e.nextElement();

      String[]     names = splitZipEntryName( entry.getName() );

      SortedTreeNode parent = checkCreateDirectoryNode( names[0] );
      checkCreateFileNode( names[0], names[1], ziproot );

    }
    return ziproot;
  }



  /**
   * check in our pathToNodeTable whether we already have a tree node
   * corresponding to 'pathname'. If so, return the node.
   * If not, create the necessary entries and its parent entries.
   */
  public SortedTreeNode checkCreateDirectoryNode( String pathname ) {
    //msg( "... cCDN '" + pathname + "'" );
    if (pathname == null) { // should not happen... replace with empty String
      msg( "-W- internal in checkCreateDirectoryNodes: pathname is null..." );
      pathname = "";
    }

    // check whether we have a valid node for 'pathname'
    //
    SortedTreeNode node = (SortedTreeNode) pathToNodeTable.get( pathname );
    if (node != null) return node;

    // pathname is not in our hashtable yet, recurse to check parent pathname
    //
    String[] names = splitZipEntryName( pathname );
    SortedTreeNode parent  = checkCreateDirectoryNode( names[0] ); 

    // now create a new tree node and register it to the hashtable
    //
    SortedTreeNode newnode = new DirectoryTreeNode(); // FIXME
    newnode.setUserObject( stripDirectoryNames(pathname) );

    pathToNodeTable.put( pathname, newnode );
    //msg( "... cCDN: created new pathname node: '" + pathname + "'" );

    return parent;
  }


  public void checkCreateFileNode( String pathname, String name, 
                                   SortedTreeNode zipRoot ) {
    SortedTreeNode fileNode, pathNode;

    if (isDesignFile(name)) {
       fileNode = createLeafNode( stripDirectoryNames(name) ); // FIXME
       pathNode = (SortedTreeNode) pathToNodeTable.get( pathname );

       pathNode.add( fileNode );
       //msg( "cCFN: added '" + name + "' to '" + pathNode + "'" );
       checkLinkParentNodes( pathname, zipRoot );
    }
  }

 
  public void checkLinkParentNodes( String pathname,
                                    SortedTreeNode zipRoot) 
  {
    SortedTreeNode node = (SortedTreeNode) pathToNodeTable.get( pathname );
    if (node.getParent() != null) return; // already linked
    if (node == zipRoot) return; // don't (re-) link the root node

    String[] names = splitZipEntryName( pathname );
    checkLinkParentNodes( names[0], zipRoot );

    // when we arrive here, our parent nodes have been linked.
    // Now link the current node.
    SortedTreeNode parent = (SortedTreeNode) pathToNodeTable.get( names[0] );
    parent.add( node );
  }


  /**
   * split a zip entry name into the path and the filename with extension.
   * This allows us to check and build the directory tree nodes fast.
   * The method returns an array of Strings with the result[0] = pathname
   * and result[1] = filename.
   * This method also strips of a leading '/', if any.
   */
  public String[] splitZipEntryName( String name ) {
    String result[] = new String[2];
    int pos = name.lastIndexOf( '/' );
    if (pos <= 0) { // no subdirectories
      result[0] = "";
      result[1] = name;
    }
    else {
      if (name.charAt(0) == '/') result[0] = name.substring( 1, pos );
      else                       result[0] = name.substring( 0, pos );
      result[1] = name.substring( pos+1, name.length() );
    }
    //msg( "... sZEN: '" + name + "' '" + result[0] + "' '" + result[1] + "'" );
    return result;
  }
 
 
  /* ********************************************************************* */
  /* ********************************************************************* */
  /* ***** Tree Path Stuff *********************************************** */
  /* ********************************************************************* */
  /* ********************************************************************* */


  public static String getFullPathname( SortedTreeNode node ) {
    if (node == null) return null;

    Object[] userObjectPath = node.getUserObjectPath();
    StringBuffer sb = new StringBuffer( "" );

    // don't include "root" node at index 0
    //
    for( int i=1; i < userObjectPath.length; i++ ) {
      sb.append( userObjectPath[i] );
      if (i < userObjectPath.length-1) sb.append( "/" );
    }
    return sb.toString();
  }


  /**
   * construct the pathname for a GIF icon corresponding to 'pathname':
   * we strip off the extension and replace it by ".gif".
   * FIXME: this should probably use a better algorithm.
   */
  public String getGifPathame( String pathname ) {
    try {
      int pos = pathname.lastIndexOf( '.' );
      return pathname.substring( 0, pos ) + ".gif";
    }
    catch( Exception e ) {
      return "/hades/manager/icons/null.gif";
    }
  }



  public void registerZipFile( String filename ) {
    try {
      ZipFile zipfile = new ZipFile( filename );
      DesignManager.getDesignManager().registerZipFile( zipfile );
      ZipRootTreeNode ziproot = this.traverseZip( zipfile );
      this.getRootNode().add( ziproot );
    }
    catch( Exception e) {
      msg( "-E- BrowserFrame.registerZipFile failed: " + filename );
      msg( "-E- exception is: " + e );
    }
  }



  /* ********************************************************************* */
  /* ********************************************************************* */
  /* ***** TESTING ******************************************************** */
  /* ********************************************************************* */
  /* ********************************************************************* */


  public void testFileSystem( String filename, SortedTreeNode root ) {
    msg( "testing: file system " + filename + "..." );
    msg( "scanning ..." );
    traverseFileTree( new File( "." ), root );
    msg( "ok..." );
  }



  public void dumpTree( SortedTreeNode root ) {
    msg( "\nprinting: full tree..." );
    Enumeration e = root.depthFirstEnumeration();
    while( e.hasMoreElements() ) {
      SortedTreeNode tmp = (SortedTreeNode) e.nextElement();
      msg( tmp.toString() + " " + tmp.isLeaf() );
    }
  }


  public void displayTree( SortedTreeNode root ) {
    JTree tree = new JTree();

    tree.setModel( new DefaultTreeModel( root ));
    tree.setRootVisible(false);

    // set TreeCellRenderer (used to draw nodes)
    tree.setCellRenderer(new HadesTreeCellRenderer());
    tree.putClientProperty("JTree.lineStyle", "Angled");

    // how many Components can be selected at one time?
    DefaultTreeSelectionModel dtsm = new DefaultTreeSelectionModel();
    dtsm.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.setSelectionModel(dtsm);

    JScrollPane scroller = new JScrollPane( tree );

    java.awt.Frame frame = new java.awt.Frame( "TreeManager selftest" );
    frame.add( "Center", scroller );
    frame.setSize( 300, 700 );
    frame.show();

    TreePopupMenu popup = new TreePopupMenu( tree );
    tree.addTreeSelectionListener( new TSL() );
  }


  /* ********************************************************************* */
  /* ********************************************************************* */
  /* ***** MAIN ********************************************************** */
  /* ********************************************************************* */
  /* ********************************************************************* */


  public static void msg( String s ) {
    System.out.println( s );
  }


  public static void main( String argv[] ) throws Exception {

    TreeManager tb = new TreeManager();
    tb.setRootNode( new RootTreeNode() );

    //tb.testFileSystem( argv[0], root );
    //tb.dumpTree( root );


    ZipFile zipfile = new ZipFile( argv[0] );   
    DesignManager.getDesignManager().registerZipFile( zipfile );
    ZipRootTreeNode ziproot = tb.traverseZip( zipfile );
    tb.getRootNode().add( ziproot );
    //tb.dumpTree( root );

/*
    SortedTreeNode ziproot2 = new ZipRootTreeNode();
    ziproot2.setUserObject( argv[1] ); // FIXME
    tb.getRootNode().add( ziproot2 );
    tb.testZip( argv[1], ziproot2 );
    //tb.dumpTree( root );
*/

    SortedTreeNode pwd = new FileRootTreeNode();
    pwd.setUserObject( "." ); // FIXME
    tb.getRootNode().add( pwd );
    tb.testFileSystem( ".", pwd );
    

    tb.displayTree( tb.getRootNode() );
  }
}

