/* SortedTreeNode - hades.manager.tree.SortedTreeNode
 *
 * subclass of DefaultMutableTreeNode with alphabetical sorting
 *
 * 04.03.01 - added handleTreeSelection() and handleTreeDoubleClick()
 * 01.03.01 - added getIcon() and getPath()
 * 27.02.01 - integration into Hades 0.87
 * 01.07.98 - original version of TNode by
 * @author   Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author   Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager.tree;

import java.net.URL;                         // workload 
import java.util.*;
import javax.swing.ImageIcon;
import javax.swing.tree.*;

import hades.manager.*;


/**
 * SortedTreeNode: a mutable tree node which sorts its children alphabetically.
 */
public class SortedTreeNode 
       extends  DefaultMutableTreeNode {

  static boolean debug = false;
  static ImageIcon _icon = loadIcon( "/hades/manager/icons/unknown.gif" ); 
  URL  __url;


  public SortedTreeNode() {
    super();
  }

  public SortedTreeNode( Object userObject ) {
    super( userObject );
  }


  /**
   * this is a base class without icon: return the "unknown" icon
   */
  public ImageIcon getIcon() {
    return _icon;
  }


  public static ImageIcon loadIcon( String resourcename ) {
   try {
     return new ImageIcon(
       ImageIcon.class.getResource( resourcename ));
    }
    catch( Exception e ) {
      System.out.println( "-E- " + e + " " + resourcename );
      return null;
    }
  }


  public URL getURL() {
    return __url;
  }


  public void setURL( URL u ) {
    __url = u;
  }


  /** adds a TreeNode to the child list */
  public synchronized void add( SortedTreeNode child ) {
//    super.add( child ); 
//  }
//
// public void _add( SortedTreeNode child ) {

    // System.out.println( "-#- add: " + this + " " + child );

    if (getChildCount() == 0) { // first child: let our base class work
      super.add( child ); 
      child.setParent(this);
      return;
    }

    child.setParent(this);

    // get name of new SortedTreeNode
    String name = child.toString().toLowerCase();   

    int i = 0;
    if (child.isLeaf())
    {
      // insert leaf node
      // skip directories
      while ( (i<getChildCount())
               &&(!((SortedTreeNode)children.elementAt(i)).isLeaf()))
      { i++; }

      // search alphabetical order
      while ((i<children.size())
         &&(name.compareTo(children.elementAt(i).toString().toLowerCase())>=0))
      { i++; }
      children.insertElementAt(child,i);
    }
    else
    {
      // insert directory
      if (child.isLocal())
      {
        // search alphabetical order
        while ((i<children.size())&&
          (name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)&&
            (!((SortedTreeNode)children.elementAt(i)).isLocal()))
        { i++; }
      }
      else if (child.isServer())
      {
        // skip 'locals'
        while ((i<children.size())&&(((SortedTreeNode)children.elementAt(i)).isLocal()))
        { i++; }
        // search alphabetical order
        while ((i<children.size())&&
          (name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)&&
            (!((SortedTreeNode)children.elementAt(i)).isServer()))
        { i++; }
      }
      else if (child.isResult())
      {
        // skip 'locals'
        while ((i<children.size())&&(((SortedTreeNode)children.elementAt(i)).isLocal()))
        { i++; }
        // skip server/bookmarks
        while ((i<children.size())&&(((SortedTreeNode)children.elementAt(i)).isServer()))
        { i++; }
        // search alphabetical order
        while ((i<children.size())&&
          (name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)&&
            (!((SortedTreeNode)children.elementAt(i)).isLeaf()))
        { i++; }
      }
      else
      {
        // search alphabetical order
        while ((i<children.size())&&
          (name.compareTo(children.elementAt(i).toString().toLowerCase())>=0)&&
            (!((SortedTreeNode)children.elementAt(i)).isLeaf()))
        { i++; }
      }
      children.insertElementAt(child,i);
    }
  }
 
 
  
  /** Returns true if user-object is "local" or "system" */
  private boolean isLocal()
  {
    return ((userObject instanceof String) && 
      (((String)userObject).equals("local")||
      ((String)userObject).equals("system")));
  }
  
  /** Returns true if user-object is 'search ...' (Search Result) */
  private boolean isResult()
  {
    return ((userObject instanceof String) && 
      (((String)userObject).startsWith("search")));
  }
  
  /** Returns true if user-object is a bookmark (server) */
  private boolean isServer()
  {
    //return (userObject instanceof Bookmark);
    return false;
  }


  /**
   * handle a tree selection event on this node in the Colibri tree control. 
   * This is a base class: we reset the browser's imageCanvas icon
   * and all InfoPanel data...
   */
  public void handleTreeSelection( BrowserFrame browser ) {
    msg( "-#- STN.handleTreeSelection: " + toString() ); 
    browser.setThumbnail( null );
    InfoPanel ip = browser.getInfoPanel();
    ip.setName( toString() );
    ip.setAuthor( "-" );
    ip.setVersion( "-" );
    ip.setInfoText( "" );
  }


  
  /**
   * handle a tree double click event on this node in the Colibri tree control. 
   * This is a base class: we do nothing...
   */
  public void handleDoubleClick( BrowserFrame browser ) {
    msg( "-#- STN.handleDoubleClick ignored: " + toString() + " " +
      TreeManager.getFullPathname( this ) );
  }


  /**
   * utility method to print a String, if 'debug' is true.
   */
  public static void msg( String s ) {
    if (debug) System.out.println( s );
  }
}
